import os
import boto3
import logging
from lantern_sl.utils.request import http_response, http_error

s3 = boto3.resource('s3')
log = logging.getLogger(__name__)


class FileUploadController(object):
    """ Handles all operations related to Upload files to S3
    """

    def __init__(self, bucket_name, static_url=None, debug=False):
        """ Constructor

        Arguments:
            bucket_name {str} -- Destination Bucket Name

        Keyword Arguments:
            static_url {str} -- Custom Static Url, It can be a dns used for the bucket (default: {None})
                If static_url==None a default static_url will be initialized for us-wes-2
            debug {bool} -- if set to True will raise any exception without handling it (default: {False})
        """
        self.debug = debug if debug else os.getenv("LOCAL_USER", False)
        self.bucket_name = bucket_name
        self.static_url = static_url if static_url else "https://s3-us-west-2.amazonaws.com/%s/" % bucket_name


    def upload_stream(self, stream, filename, mimetype, return_raw=False):
        """ Return a file stream, this is a manual operation use upload_file and upload_image instead

        Arguments:
            stream {FileStorage or any other file opened} -- File Stream to be uploaded to s3
            filename {str} -- Destination filename
            mimetype {str} -- mimetype without charset

        Keyword Arguments:
            return_raw {bool} -- If set to True it will retunr just the url not the response (default: {False})

        Returns:
            [HttpReseponse, status or url] -- return url or httpresponse,status depending on return_raw
        """
        try:
            s3.Bucket('iotsigfox-lantern-client-staticfiles').put_object(
                Key=filename,
                Body=stream,
                ContentType=mimetype)
            final_url = "{}{}".format(self.static_url, filename)
            return final_url if return_raw else http_response(code=code, message="Uploaded", data=final_url)
        except Exception as e:
            log.error(str(e))
            if self.debug:
                raise e
            elif return_raw:
                return None
            else:
                http_error(code=500, message="Error trying to upload the file", detail=str(e))


    def upload_file(self, stream, filename, mimetype=None, return_raw=False):
        """ Upload files without checking extention

        Arguments:
            stream {FileStorage or any other file opened} -- File Stream to be uploaded to s3
            filename {str} -- Destination filename
            mimetype {str} -- mimetype without charset

        Keyword Arguments:
            return_raw {bool} -- If set to True it will retunr just the url not the response (default: {False})

        Returns:
            [HttpReseponse, status or url] -- return url or httpresponse,status depending on return_raw
        """
        mimetype = mimetype if mimetype else stream.mimetype
        return self.upload_stream(stream=stream, filename=filename, mimetype=mimetype, return_raw=return_raw)


    def upload_image(self, stream, filename, mimetype=None, return_raw=False):
        """ Upload Images to S3
            We are validating image is present in the mimetype to make sure a supported image is being uploaded.
            
            Arguments:
                    stream {FileStorage or any other file opened} -- File Stream to be uploaded to s3
                    filename {str} -- Destination filename
                    mimetype {str} -- mimetype without charset
                
                Keyword Arguments:
                    return_raw {bool} -- If set to True it will retunr just the url not the response (default: {False})
                
                Returns:
                    [HttpReseponse, status or url] -- return url or httpresponse,status depending on return_raw
        """

        mimetype = mimetype if mimetype else stream.mimetype
        if "image" not in mimetype:
            message = "{} not supported for image upload".format(mimetype)
            detail = "A image extention is required"
            if self.debug:
                raise Exception(message)
            elif return_raw:
                return None
            else:
                return http_error(code=415, message=message, detail=detail)
        return self.upload_stream(stream=stream, filename=filename, mimetype=mimetype, return_raw=return_raw)


import os
import json

class CognitoController(object):
    """ Allow us to easily fetch cognito parsed data 
    """

    def __init__(self, debug=False):
        self.debug = debug if debug else os.getenv("LOCAL_USER", False)

    def get(self, attribute_name, request, json_parse=False):
        """ retrieve a specific attribute_name from cognito headers
        
        Arguments:
            attribute_name {str} -- attribute name to be searched in the header
            request {flask.request} -- Active request, in http request.
        
        Keyword Arguments:
            json_parse {bool} -- force a json parse if active (default: {False})
        
        Raises:
            Exception -- generic exception, example: no request was provided
            ValueError -- Any Value error, example: attribute_name not found in header
        
        Returns:
            [str | json] -- if json_parse is False, it returns the scalar value, if it's activated we return the json object. 
        """

        if not request:
            raise Exception(
                "You have to specify a valid `request` parameter (inside a http request)")
        if self.debug:
            _local_header = {
                "authorizer": '{"claims": {"custom:fullName": "local_user", "cognito:username": "local_user", "email": "local_user@lantern.tech", "email_verified": true}}',
                "Authorization": "{}".format(os.getenv("LOCAL_TOKEN", ""))
            }
            header_value = _local_header.get(attribute_name, None)
        else:
            header_value = request.headers.get(attribute_name, None)
        if not header_value:
            raise ValueError("{} not found in headers".format(attribute_name))

        if not json_parse:
            return header_value
        else:
            header_value = header_value.replace("'", '"')
            header_value_json = json.loads(header_value)
            return header_value_json


    def get_default(self, name, request, header_attribute_name="authorizer", container_node="claims"):
        """ Return a value or json using de default mapping
        
        Arguments:
            name {str} -- attribute name to be searched in header
            request {flask.request} -- request in active HTTP request
        
        Keyword Arguments:
            header_attribute_name {str} -- header key containing cognito data (default: {"authorizer"})
            container_node {str} -- second level node, containing cognito data (default: {"claims"})

        Returns:
            [str] -- Value in `request.header_attribute_name.container_node.name`
        """

        header_value = self.get(
            attribute_name=header_attribute_name, request=request, json_parse=True)
        if container_node:
            header_node = header_value.get(container_node, None)
            if not header_node:
                raise ValueError("'{}' container_node not found in header (inside: {})".format(
                    container_node, header_attribute_name))
            requested_value = header_node.get(name, None)
            if not requested_value:
                raise ValueError(
                    "'{}' value empty/not found in {}.{}".format(name, header_attribute_name, container_node))
            return requested_value

        authorizer = request.headers.get(header_attribute_name, None)
        if not authorizer:
            raise Exception('''
             It was impossible to find cognito authorizer in heders, check your zappa_settings.json
             Example:
             "authorizer": {
                    "type": "COGNITO_USER_POOLS",
                    "provider_arns": [
                        "arn:aws:cognito-idp:us-west-2:XXXXXXXXXXX:userpool/us-west-2_XXXXXXXXX"
                    ]
                },
                "context_header_mappings": {
                    "authorizer" : "authorizer"
                }
            ''')



'''
from lantern_flask.controllers.cognito_controller import CognitoController
from lantern_flask.controllers.dynamo_controller import (
    DynamoController,
    ExceptionInvalidDynamoControllerType,
    ExceptionInvalidDynamoControllerNotFound,
)
from lantern_flask.controllers.fileupload_controller import FileUploadController
from lantern_flask.controllers.application_controller import ApplicationController
'''